;;; -*- Mode: Emacs-Lisp; Mode: Linkd -*-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;; Emacs Customizations -- Christopher R. Genovese           ;;
;;                                                           ;;
;; Version 2.4  Last Updated 24 Oct 2010                     ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; NOTE: If you have (@url :file-name "http://www.emacswiki.org/emacs/LinkdMode" :display "linkd.el"), you can engage \\[linkd-mode]
;;       to convert the table of contents below into links 
;;       throughout this file.


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; TABLE OF CONTENTS

;;  1. (@> "Emacs Lisp Utility Functions")
;;  2. (@> "Environment Configuration")
;;  3. (@> "Imported Mode Requirements")
;;  4. (@> "Text Modes")
;;  5. (@> "Programming Modes")
;;  6. (@> "Shell-based Modes")
;;  7. (@> "File and Navigation Modes")
;;  8. (@> "TeX-related Modes")
;;  9. (@> "ESS Mode for R and S Code")
;; 10. (@> "Global Key Bindings")
;; 11. (@> "Settings and Initialization")
;; 12. (@> "System Dependent Settings")
;; 13. (@> "Faces")
;; 14. (@> "Initial State")



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Emacs Lisp Utility Functions")                        ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun make-backup-file-name (file)
  "Create the non-numeric backup file name for FILE.
This is a separate function so you can redefine it for customization."
  (concat (file-name-directory file) (concat "." (concat (file-name-nondirectory file) "~"))))

(defun set-abbrev-mark ()
  "Mark current point as the beginning of an abbrev.
Abbrev to be expanded starts here rather than at beginning of word before point."
  (interactive)
  (setq abbrev-start-location (point-marker)
	abbrev-start-location-buffer (current-buffer))
  (message "Abbrev Mark Set"))

;; Frame functions

(defun my-other-frame (arg)
  "Select the ARG'th different visible frame, and raise it, even if
the frame is iconified or invisible. All frames are arranged in a
cyclic order, and this command selects the frame ARG steps away in
that order.  A negative ARG moves in the opposite order."
  (interactive "p")
  (let ((frame (selected-frame)))
    (while (> arg 0)
      (setq frame (next-frame frame))
      (setq arg (1- arg)))
    (while (< arg 0)
      (setq frame (previous-frame frame))
      (setq arg (1+ arg)))
    (raise-frame frame)
    (select-frame frame)
    (set-mouse-position (selected-frame) (1- (frame-width)) 0)
    (if (fboundp 'unfocus-frame)
	(unfocus-frame))))

;; Scrolling and Cursor Movement functions

(defun scroll-down-or-beg ()
  "Smart scroll down, if too close to the beginning of the buffer,
it moves you there without error, in contrast to scroll-down."
  (interactive)
  (condition-case nil (scroll-down) (error (goto-char (point-min)))))

(defun scroll-up-or-end ()
  "Smart scroll up, if too close to the end of the buffer,
it moves you there without error, in contrast to scroll-up."
  (interactive)
  (condition-case nil (scroll-up) (error (goto-char (point-max)))))


;; Region Functions

  ;; Standard version is silent, so it is hard to tell if the key was hit
  ;; Only added a message

(defun my-copy-region-as-kill (beg end)
  "Save the region as if killed, but don't kill it.
If `interprogram-cut-function' is non-nil, also save the text for a window
system cut and paste.  Produces a message to confirm copy."
  (interactive "r")
  (if (eq last-command 'kill-region)
      (kill-append (buffer-substring beg end) (< end beg))
    (kill-new (buffer-substring beg end)))
  (message "Region Copied")
  nil)

;; Import and Import-Conditional Execution

(defmacro require-soft (feature &optional file)
  "*Try to require FEATURE, but don't signal an error if `require' fails."
  `(require ,feature ,file 'noerror)
  )

(defmacro with-library (symbol &rest body)
   "*Execute BODY conditionally on the availability of library SYMBOL"
   `(if (not (require ,symbol nil t))
        (message (format "Library %s is not available." ',symbol))
      ,@body))

;; List Matching Functions

(defun remove-matching-elements (elt lst &optional pred)
  "Remove elements in LST matching ELT according to the
predicate PRED. If PRED is not supplied, it defaults to equal."
  (let ((pred (or pred (symbol-function 'equal)))
	(newlst nil)
	cmp)
    (while lst
      (setq cmp (car lst))
      (if (not (funcall pred elt cmp))
	  (setq newlst (cons cmp newlst)))
      (setq lst (cdr lst)))
    (reverse newlst)))
      
(defun remove-non-matching-elements (elt lst &optional pred)
  "Remove elements in LST matching ELT according to the
predicate PRED. If PRED is not supplied, it defaults to equal."
  (let ((pred (or pred (symbol-function 'equal)))
	(newlst nil)
	cmp)
    (while lst
      (setq cmp (car lst))
      (if (funcall pred elt cmp)
	  (setq newlst (cons cmp newlst)))
      (setq lst (cdr lst)))
    (reverse newlst)))

;; Predicates for element removal
      
(defun eq-car (l1 l2)
  "Returns t if car of l1 and l2 are eq, nil otherwise.
If l1 (or l2) are not cons cells, l1 (or l2) itself is used in the comparison.
This is useful as a predicate for removing elements from an association list;
see remove-matching-elements.  In this case, if the keys are symbols then
only the symbol of interest need be passed to remove-matching elements."
  (eq (if (consp l1) (car l1) l1) (if (consp l2) (car l2) l2)))

(defun equal-car (l1 l2)
  "Returns t if car of l1 and l2 are equal, nil otherwise.
If l1 (or l2) are not cons cells, l1 (or l2) itself is used in the comparison.
This is useful as a predicate for removing elements from an association list;
see remove-matching-elements.  In this case, if the keys are symbols then
only the symbol of interest need be passed to remove-matching elements."
  (equal (if (consp l1) (car l1) l1) (if (consp l2) (car l2) l2)))


;; File and Directory Name Functions and Predicates


(defun expand-directory-name (dir)
  "Expands a directory name as in expand-file-name but ensures
that the name ends with a '/'.  This can be used to convert
a directory name, even in relative or ~ form, to a canonical
form, for comparison, concatenation, or other purposes."
  (save-excursion
    (let ((dir (and (stringp dir) (expand-file-name dir))))
      (if (and dir (not (string-match "/$" dir)))
	  (concat dir "/")
	dir))) )

(defun equal-file-name (fn1 fn2 &optional func)
  "Tests whether two file-names FN1 and FN2 refer to the same file.
This uses FUNC to bring both file-names to a common form (e.g., abolute).
If FUNC is not provided, it defaults to a expand-file-name."
  (if (and (stringp fn1) (stringp fn2))
      (let ((func (or func (symbol-function 'expand-file-name))))
	(string-equal (funcall func fn1) (funcall func fn2)))))

(defun equal-dir-name (fn1 fn2 &optional func)
  "Tests whether two file-names FN1 and FN2 refer to the same file.
This uses FUNC to bring both file-names to a common form (e.g., abolute).
If FUNC is not provided, it defaults to a function that does
expand-file-name followed by appending a trailing slash."
  (if (and (stringp fn1) (stringp fn2))
      (let ((func (or func (symbol-function 'expand-directory-name))))
	(string-equal (funcall func fn1) (funcall func fn2)))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Environment Configuration")                           ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; This component is automatically generated from a template
;; by ./makeconfig, perl, and make. See Makefile for details.


(defvar my-user-name "genovese"
  "*If non-nil, this should be set to a string giving the user name
to use by default in locating the ....  If set to nil, this will
always use the HOME directory in the environment if it is available
as the root for the startup information.  Set this is you wish to
load your startup file when running as someone else via the -u flag.")

(defvar my-home-aliases (list "/Users/genovese/")
  "*This is a list of strings indicating possible aliases for the users
HOME directory that emacs might not recognize.  The first element of the list should be the
prefered or most common or most specific form.  This variable is most useful with
cross-mounted file systems, and is used by functions to determine whether 
or not a directory name should be changed to \"~/\".")

(defvar my-home-dir (if (and (equal my-user-name (getenv "USER")) (getenv "HOME"))
                        (concat (getenv "HOME") "/")
                      (or  (car my-home-aliases) ""))
  "Home directory")

(defvar my-site-lisp-dir (expand-directory-name "/Applications/Emacs.app/Contents/Resources/site-lisp/")
  "The site-lisp directory (absolute path) containing the globally available
   add-on modes and .el files. This name must end in /.")

(defvar my-home-lisp-dir (expand-directory-name "~/.emacs.d/")
  "The site-lisp directory (absolute path) containing the users local
   add-on modes and .el files. This name must end in /.")

(defvar my-frame-alist '((top . 1) (left . 1) (width . 180) (height . 62))
  "Geometry parameters that will be used for initial-frame-alist to determine initial window size.")

(if (and (boundp 'my-home-dir) (stringp my-home-dir)
         (not (string= my-home-dir "")) (not (member my-home-dir my-home-aliases)))
    (setq my-home-aliases (cons my-home-dir my-home-aliases)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Imported Mode Requirements")                          ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;
;; Standard packages whose variables/functions I use
;; 

(require 'comint)
(require 'shell)
(require 'calc)

;;
;; Built-in Extensions
;;

(require 'dired-x)
(require 'dired-aux)
(require 'org-install)
(require 'icicles)
(require 'ido)

(autoload 'nxml-mode "nxml/autostart.el" "nXML Mode" t)
(autoload 'nxhtml-mode "nxml/autostart.el" "nXHTML Mode" t)

;;
;; Third-party modes
;;

(require 'dired+)
(require 'bookmark+)
(require 'linkd)
(require 'lacarte)
(require 'ipython)
(require 'dropdown-list)
(require 'yasnippet)
(require 'anything-config)
(eval-after-load "thingatpt"
  '(require 'thingatpt+))
;(require 'ess-site)  ; loaded in ESS.el for timing purposes

;;
;; CEDET-Based IDE Modes
;;

;;CEDET
(if (not (featurep 'cedet))
      (load-file (concat my-site-lisp-dir "cedet/common/cedet.el")))
;(add-to-list 'load-path (expand-file-name (concat my-site-lisp-dir "jde/lisp")))
;?;(require 'semantic-sb)

;;JDE
(setq defer-loading-jde t)
(if defer-loading-jde
    (progn
      (autoload 'jde-mode "jde" "JDE mode." t)
      (setq auto-mode-alist
	    (append
	     '(("\\.java\\'" . jde-mode))
	     auto-mode-alist)))
  (require 'jde))


;; ECB
;(add-to-list 'load-path (expand-file-name (concat my-site-lisp-dir "ecb")))
(eval-when-compile
    (if (not (featurep 'cedet))
        (require 'cedet))
    )
(require 'ecb)  ;; or (require 'ecb-autoloads) to start with ecb-activate command

;;
;; Automatic Mode Detection
;;

(add-to-list 'auto-mode-alist '("\\.org\\'" . org-mode))
(add-to-list 'auto-mode-alist '("\\.txt\\'" . org-mode))
(add-to-list 'auto-mode-alist '("\\.[0-9]\\'" . fundamental-mode))
(add-to-list 'auto-mode-alist '("\\.[sS]\\'" . S-mode))
(add-to-list 'auto-mode-alist '("\\.[rR]\\'" . R-mode))
(add-to-list 'auto-mode-alist '("\\.\\([pP]\\([LlmM]?\\|od\\|erl\\)\\|al\\)\\'" . perl-mode))
(add-to-list 'auto-mode-alist 
             (cons (concat "\\." (regexp-opt '("xml" "xsd" "sch" "rng" "xslt" "svg" "rss") t) "\\'")
                   'nxml-mode))
(add-to-list 'auto-mode-alist '("\\.s?html?\\'" . nxhtml-mode))

(fset 'xml-mode 'nxml-mode)
(fset 'html-mode 'nxhtml-mode)

(add-to-list 'magic-mode-alist '("<\\?xml " . nxml-mode))
(add-to-list 'magic-mode-alist '("#!.*perl" . perl-mode))
(add-to-list 'magic-mode-alist '("#!.*python" . python-mode))
(add-to-list 'magic-mode-alist '("#!.*\\(t?c\\|ba\\|k\\|z\\|\\)sh" . shell-script-mode))
(add-to-list 'magic-mode-alist '("\\\\input " . plain-TeX-mode))
(add-to-list 'magic-mode-alist '("\\\\document\\(style\\|class\\)" . LaTeX-mode))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Text Modes")                                          ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Org Mode

(defun my-org-load-hook ()
  (setq org-startup-folded nil)
  )

(defun my-org-mode-hook ()
  (local-set-key "\C-c\C-x\M-k" 'org-cut-special)
  (local-set-key "\C-c\C-x\M-c" 'org-copy-special)
  )

(add-hook 'org-load-hook 'my-org-load-hook)
(add-hook 'org-mode-hook 'turn-on-font-lock)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Programming Modes")                                   ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; IDE Configuration CEDET/ECB/JDEE  (necessary files loaded in imports.el)

(add-hook 'semantic-init-hook 'senator-minor-mode)
(add-hook 'semantic-init-hook
          (lambda ()
            (global-semantic-idle-scheduler-mode '1)
            (semantic-load-enable-excessive-code-helpers)
            ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Emacs-Lisp Mode

(add-hook 'emacs-lisp-mode-hook
	  (function (lambda ()
		      (local-set-key "\C-c\C-e" 'eval-defun)
		      (local-set-key "\C-c\C-o" 'outline-minor-mode)
		      (local-set-key "\C-c\C-s" 'eval-last-sexp)
		      (defun emacs-lisp-outline-minor-setup ()
			(setq outline-regexp ";;; \\|;; \\|(....")
			(local-set-key "\C-c\C-m" outline-mode-prefix-map))
                      (make-local-variable 'outline-minor-mode-hook)
                      (add-hook 'outline-minor-mode-hook 'emacs-lisp-outline-minor-setup)
		      )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Common C Modes (cc-mode) (C, C++, Java)

(autoload 'c-mode    "cc-mode" "C Editing Mode" t)    ;; Use cc-mode for all C editing
(autoload 'c++-mode  "cc-mode" "C++ Editing Mode" t)
(autoload 'java-mode "cc-mode" "Java Editing Mode" t)

(defun my-c-mode-common-hook ()
  (if (not (assoc "CRG" c-style-alist))
      (c-add-style "CRG" (cdr c-crg-style)))
  (c-set-style "CRG")
  (if (assq 'c-auto-hungry-string minor-mode-alist)   ;; Inhibit minor mode notation
      (setq minor-mode-alist
	    (cons '(c-auto-hungry-string "") (remove-matching-elements 'c-auto-hungry-string minor-mode-alist 'eq-car))))
  (local-set-key "\C-c\C-a" 'c-beginning-of-statement)
  (local-set-key "\C-c\C-e" 'c-end-of-statement)
  (local-set-key "\C-c\C-b" 'c-backward-conditional)
  (local-set-key "\C-c\C-f" 'c-forward-conditional)
  (local-set-key "\C-c\C-c" 'comment-region)
  (local-set-key "\M-q"     'c-fill-paragraph)
  (local-set-key "\C-c\C-d" 'c-down-conditional)
  (local-set-key "\C-c\C-u" 'c-up-conditional)
  (local-set-key "\C-c\C-i" 'c-indent-exp)
  (local-set-key "\C-c\C-q" 'c-indent-defun)
  (local-set-key "\C-c\C-m" 'compile);; m for make
  (local-set-key "\C-c\C-n" 'up-list)
  (local-set-key "\C-c\C-p" 'c-up-list-neg)
  (local-set-key "\C-c\C-o" 'outline-minor-mode)
  (local-set-key "\C-c\C-t" 'c-set-offset)
  (local-set-key "\C-c\C-x" 'c-macro-expand)
  (local-set-key [f4]       'c-fill-paragraph)
  )
  
(add-hook 'c-mode-common-hook 'my-c-mode-common-hook)

(defvar c-crg-style
  '("CRG"
    (c-basic-offset . 4)
    (c-comment-only-line-offset . (0 . 0))
    (c-offsets-alist . ((statement-block-intro . +)
			(substatement-open . 0)
			(label . 0)
			(case-label . *)
			(access-label . /)
			(statement-case-intro . *)
			(statement-cont . +)
			(cpp-macro . 0)
                        (inline-open . 0)
			(inher-intro . c-lineup-inher-intro)
			))
    (c-auto-newline . t)
    (c-hanging-braces-alist . ((brace-list-open)
			       (brace-list-close)
			       (substatement-open before after)
			       (block-close before after)))
    (c-hanging-comment-ender-p . nil)
    (c-cleanup-list . '(scope-operator list-close-comma))
    )
  "My personal formatting style for editing C/C++/Java source.")

(defun c-up-list-neg (arg)
  "Like up-list but with a negated arg"
  (interactive "P")
  (if (not arg)
      (up-list -1)
    (up-list arg)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; C-mode (old)

(setq c-mode-hook
      '((lambda ()
	  (setq indent-tabs-mode nil)
	  (local-set-key "\C-c\C-z" 'compilation-close-compilation)
	  )))

(defvar boc-mode-hook                    
  '((lambda ()
      (setq c-indent-level '4)
      (setq c-continued-statement-offset '4)
      (setq c-continued-brace-offset '-4)
      (setq c-brace-offset '0)
      (setq c-brace-imaginary-offset '0)
      (setq indent-tabs-mode 'nil) 
      (setq c-label-offset '-2)
      (setq c-auto-newline t)
      ))
  "Style hook for \"Boring Old C-mode\" in case
it is needed to run an earlier version of emacs.")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; C++-mode

(setq c++-mode-hook
      '((lambda ()
	  (setq indent-tabs-mode nil)
	  (local-set-key "\C-c\C-z" 'compilation-close-compilation)
	  )))

(defun c-lineup-inher-intro (langelem)
  "Line up inheritence intro : two characters past class keyword"
  (+ c-basic-offset (/ c-basic-offset 2))) 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; java-mode

(setq java-mode-hook
      '((lambda ()
	  (setq indent-tabs-mode nil)
	  )))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; compilation-mode

(setq compilation-mode-hook
      '((lambda ()
	  (local-set-key "\C-c\C-z" 'compilation-close-compilation)
	  )))

(defun compilation-close-compilation (arg)
  "Iconifies compilation pop-up frame.  With prefix arg,
kills *compilation* buffer and thus its dedicated frame,
if the latter exists."
  (interactive "P")
  (let* (err-msg buf)
    (setq buf (condition-case err-msg
		  (setq buf (compilation-find-buffer))
		(error (and (message (car (cdr err-msg))) nil))))
    (if (and buf (not arg))
	(let ((win (get-buffer-window buf t)))
	  (if win
	      (iconify-frame (window-frame win))))
      (if (and buf arg)
	  (kill-buffer buf))
      )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; perl-mode

(setq perl-mode-hook
      '((lambda ()
	  (setq perl-indent-level '4)
	  (setq perl-continued-statement-offset '4)
	  (setq perl-continued-brace-offset '-4)
	  (setq perl-brace-offset '0)
	  (setq perl-brace-imaginary-offset '0)
	  (setq indent-tabs-mode 'nil) 
	  (setq perl-label-offset '-2)
	  )))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Shell-based Modes")                                   ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; ipython

(require-soft 'ipython)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; comint-mode 

;; (autoload 'my-comint-bol "my-comint-ext")  ;; ATTN: I think this is right to remove

(add-hook 'comint-mode-hook
      (function (lambda ()
	  (setq comint-input-ring-size '1024)
	  (setq comint-input-ignoredups t)
          (add-hook 'comint-output-filter-functions 'comint-strip-ctrl-m)
	  )))
       

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Extension to comint-mode, provides a few convenient functions
;;;

(require 'comint)

(defun my-comint-bol ()
  "Goes to the beginning of line, then skips past the prompt, if any.
However, if already at the prompt, the prompt is not skipped.
This has the side-effect of skipping the prompt if you start at the
beginning of a line with a prompt.  This oscillation between the two
possible beginnings is useful.

The prompt skip is done by skipping text matching the regular expression
`comint-prompt-regexp', a buffer local variable."
  (interactive)
  (let ((now (point))
	(eol (save-excursion (end-of-line) (point))))
    ;; (beginning-of-line)  ;; ATTN: this got confused by "field boundaries" in later versions of emacs
    (forward-line 0)
    (if (looking-at comint-prompt-regexp)
	(if (and (not (= (match-end 0) now))
		 (<= (match-end 0) eol))
	    (goto-char (match-end 0))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; shell-mode

(setq-default shell-file-name "/bin/tcsh")

(add-hook 'shell-mode-hook
      (function (lambda ()
          ;; (require 'my-shell-ext)
	  (local-set-key "\C-c\C-q" 'shell-toggle-autoexpand)
	  (local-set-key "\C-a" 'my-comint-bol)
          (setq shell-pushd-regexp "\\(pd\\|cd\\|pushd\\)")
	  (setq shell-cd-regexp    "chdir")
	  (setq shell-popd-regexp  "\\(pf\\|popd\\)")
	  (setq shell-prompt-pattern "^[^#%$>]*[#$%>]>? *" )
	  (setq shell-pushd-dextract t)
	  (setq shell-pushd-dunique t)
	  (setq shell-pushd-silent nil)
	  (setq shell-pushd-tohome t)
	  (setq comint-process-echoes nil)
	  (setq comint-input-autoexpand 'input)
	  (setq shell-input-autoexpand 'input)
	  (setq shell-backup-input-autoexpand (cons nil shell-input-autoexpand))
          ;;
	  ;; Check that starting directory is correct on startup
          ;;
	  (setq default-directory (shell-map-home-aliases default-directory))
          ;;
	  ;; Include cwd on the mode line
          ;;
	  (make-local-variable 'mode-line-format)
	  (setq mode-line-format
		(reverse (append
			  (list " -%-" 'default-directory "-- ")
			  (cdr (reverse (default-value 'mode-line-format))))))
	  ;;
	  ;; Configure Output Filter 
	  ;;
          (add-hook 'comint-output-filter-functions 'comint-watch-for-password-prompt nil t)
	  ;;
	  ;; Set-up replacements for shell.el functions, mine given below
	  ;;
          (use-my-shell-ext)
	  )))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Extension to shell-mode to improve the directory stack handling
;; and to add a few convenient features.
;;

(require 'shell)

(defvar shell-pushd-silent nil
  "*If non-nil, show dirstack in minibuffer with every change
of the directory stack.  This mimicks the optional behavior
of tcsh.")

(defvar shell-backup-input-autoexpand (cons nil 'input)
  "*List of autoexpand states among which the user can toggle with
shell-toggle-autoexpand.")

(defvar shell-home-aliases (if (boundp 'my-home-aliases)
                               my-home-aliases
                             (if (getenv "HOME")
                                 (list (concat (getenv "HOME") "/"))
                               nil))
  "*This is a list of strings indicating possible aliases for the users
HOME directory that emacs might not recognize.  This is most useful with
cross-mounted file systems, and is used by shell-map-home-aliases to 
determine whether or not the directory should be changed to \"~/\".")


(defun use-my-shell-ext ()
  "Set selected shell mode functions to my extended versions."
  (fset 'shell-resync-dirs
        (symbol-function 'my-shell-resync-dirs))
  (fset 'shell-directory-tracker
        (symbol-function 'my-shell-directory-tracker))
  (fset 'shell-process-cd
        (symbol-function 'my-shell-process-cd))
  (fset 'shell-process-pushd
        (symbol-function 'my-shell-process-pushd))
  (fset 'shell-dirstack-message
        (symbol-function 'my-shell-dirstack-message))
  (fset 'shell-replace-by-expanded-directory
        (symbol-function 'my-shell-replace-by-expanded-directory)) )

(defun shell-toggle-autoexpand ()
 "Toggle history auto-expansion to eliminate strange behavior in interactive
  applications run within the shell.  The list shell-backup-input-autoexpand
  stores the list of values to be toggled among.  At the moment, don't set
  shell/comint-input-autoexpand without changing this list as well."
  (interactive)
  (let ((nextia  (car shell-backup-input-autoexpand))
        (otheria (cdr shell-backup-input-autoexpand))
        (curia   comint-input-autoexpand))
    (setq comint-input-autoexpand nextia)
    (setq shell-backup-input-autoexpand (cons otheria nextia))
    (if nextia
       (message (format "Setting Input Autoexpand to %s" nextia))
       (message "Turning Off Input Autoexpand"))))

 ;;; Additions and fixes to shell directory tracking mechanism,
 ;;; based on the code in shell.el Gnu Emacs 19.30.1.
 ;;;
 ;;; Features:
 ;;; 
 ;;; 1. Support =n and =- references to directory stack (without expansion) as in tcsh
 ;;; 2. Fixed expansion of =n and =- (when instigated) to prevent missing slash
 ;;; 3. Sets shell-last-dir properly so  cd - works as in tcsh
 ;;; 4. Fixed bug in shell-resync-dirs in case dir tracking gets lost
 ;;; 5. Improved handling of aliases for HOME directory.


(defun shell-map-home-aliases (indir &optional noslash)
  "Check if DIR matches one of the aliases for HOME
and thus should be replaced by \"~/\" for consistency."
  (let ((dir indir))
    (if (not (string-match "/$" dir))
	(setq dir (concat dir "/")))
    (if (and shell-home-aliases (member dir shell-home-aliases))
	"~/"
      (if noslash indir dir)) ))

(defun my-shell-resync-dirs ()
  "Resync the buffer's idea of the current directory stack.
This command queries the shell with the command bound to 
`shell-dirstack-query' (default \"dirs\"), reads the next
line output and parses it to form the new directory stack.
DON'T issue this command unless the buffer is at a shell prompt.
Also, note that if some other subprocess decides to do output
immediately after the query, its output will be taken as the
new directory stack -- you lose. If this happens, just do the
command again."
  (interactive)
  (let* ((proc (get-buffer-process (current-buffer)))
	 (pmark (process-mark proc)))
    (goto-char pmark)
    (sit-for 0) ; force redisplay
    (comint-send-string proc shell-dirstack-query) 
    (comint-send-string proc "\n")
    (set-marker pmark (point))
    (let ((pt (point))) ; wait for 1 line
      ;; This extra newline prevents the user's pending input from spoofing us.
      (insert "\n") (backward-char 1)
      (while (not (looking-at ".+\n"))
	(accept-process-output proc)
	(goto-char pt))
      (if (looking-at (concat shell-dirstack-query "\n"))
	  (accept-process-output proc)))
    (goto-char pmark) (delete-char 1) ; remove the extra newline
    (forward-line -1)
    ;; That's the dirlist. grab it & parse it.
;    (let* ((dl (buffer-substring (match-beginning 0) (1- (match-end 0))))
    (let* ((dl (buffer-substring (point) (save-excursion (end-of-line) (point))))
	   (dl-len (length dl))
	   (ds nil)			; new dir stack
	   (i 0)
	   new-dir)
      (while (< i dl-len)
	;; regexp = optional whitespace, (non-whitespace), optional whitespace
	(string-match "\\s *\\(\\S +\\)\\s *" dl i) ; pick off next dir
	(setq new-dir (concat comint-file-name-prefix
			      (substring dl (match-beginning 1) (match-end 1))))
	(setq i (match-end 0))
	(if (not (string-match "/$" new-dir))
	    (setq new-dir (concat new-dir "/")))
	(setq ds (cons new-dir ds)) )
      (goto-char (process-mark proc))
      (let ((ds (nreverse ds)))
	(condition-case nil
	    (progn (shell-cd (car ds))
		   (setq shell-dirstack (cdr ds))
		   (my-shell-dirstack-message t))
	  (error (message (format "Could not cd to %s." (car ds)))))))))


(defun my-shell-directory-tracker (str)
  "Tracks cd, pushd and popd commands issued to the shell.
This function is called on each input passed to the shell.
It watches for cd, pushd and popd commands and sets the buffer's
default directory to track these commands.

You may toggle this tracking on and off with M-x dirtrack-toggle.
If emacs gets confused, you can resync with the shell with M-x dirs.

See variables `shell-cd-regexp', `shell-pushd-regexp', and `shell-popd-regexp',
while `shell-pushd-tohome', `shell-pushd-dextract' and `shell-pushd-dunique'
control the behavior of the relevant command.

Environment variables are expanded, see function `substitute-in-file-name'."
  (if shell-dirtrackp
      ;; We fail gracefully if we think the command will fail in the shell.
      (condition-case chdir-failure
	  (let ((start (progn (string-match "^[; \t]*" str) ; skip whitespace
			      (match-end 0)))
		(def-dir  default-directory)
		end cmd arg1)
	    (while (string-match shell-command-regexp str start)
	      (setq end (match-end 0)
		    cmd (comint-arguments (substring str start end) 0 0)
		    arg1 (comint-arguments (substring str start end) 1 1))
	      (cond ((string-match (concat "\\`\\(" shell-popd-regexp "\\)\\($\\|[ \t]\\)") cmd)
		     (shell-process-popd (substitute-in-file-name arg1)))
		    ((string-match (concat "\\`\\(" shell-pushd-regexp "\\)\\($\\|[ \t]\\)") cmd)
		     (shell-process-pushd (shell-map-home-aliases
					   (shell-replace-stack-dir
					    (substitute-in-file-name arg1))
					   t)))
		    ((string-match (concat "\\`\\(" shell-cd-regexp "\\)\\($\\|[ \t]\\)") cmd)
		     (shell-process-cd (shell-map-home-aliases
					(shell-replace-stack-dir
					 (substitute-in-file-name arg1))
					t))))
	      (if (equal def-dir default-directory)
		  nil
		(setq shell-last-dir def-dir)
		(setq def-dir default-directory))
	      (setq start (progn (string-match "[; \t]*" str end) ; skip again
				 (match-end 0)))))
	    (error (message (format "Could not cd: %s" (car (cdr chdir-failure))))))))

(defun shell-replace-stack-dir (str &optional nomsg defdir)
  "Replace reference to directory stack in given argument by its
value.  Leading parts of STR matching -, =n, =- (for n >= 0) are
replaced by the corresponding directory element (last dir, nth on
stack, last on stack, resp); otherwise STR itself is returned. Strings
of the form +n are not processed since, depending on variables like
'shell-pushd-dextract', these might require special treatment.
Replacement handles the addition of '/', so it is not necessary that
directory elements end in '/'.  If n above is larger than the
directory stack length, a message to that effect is printed, unless
NOMSG is non-nil.  When an ERROR occurs or when a 0 reference into the
stack is given, returns DEFDIR, which defaults to default-directory if
not provided."
  (let* ((def-dir (or defdir default-directory))
	 (stack   (cons def-dir shell-dirstack))
	 (stacklen (length stack))
	 (num (and (stringp str)
		   (or (and (string-match "^=\\([0-9]+\\)\\(/.*\\|\\s \\|$\\)" str)
			    (string-to-int (substring str (match-beginning 1) (match-end 1))))
		       (and (string-match "^=-\\(/.*\\|\\s \\|$\\)" str) (1- stacklen)))))
	 dir-head
	 dir-tail)
    (cond ((and (stringp str) (string-equal "-" str)) shell-last-dir)
	  ((null num) str)
	  ((>= num stacklen)
	   (if (not nomsg)
	       (signal 'error
		       (list (format "Directory stack not that deep (%d)." num))))
	   def-dir)
	  ((string-match "^=\\(\\([0-9]+\\|-\\)/?\\)" str) ;; assumes dirstack entries end in '/'
	   (setq dir-head (nth num stack))
	   (setq dir-tail (substring str (match-end 1)))
	   (concat dir-head (if (not (string-match "/$" dir-head)) "/") dir-tail))
	  (t str))))


;;; cd [dir]
(defun my-shell-process-cd (arg)
  (let ((new-dir (cond ((zerop (length arg)) (concat comint-file-name-prefix
						     "~/"))
		       ((string-equal "-" arg) shell-last-dir)
		       (t (shell-prefixed-directory-name arg)))))
    (shell-cd new-dir)
    (shell-dirstack-message)))

;;; pushd [+n | dir]
(defun my-shell-process-pushd (arg)
  (let ((num (shell-extract-num arg)))
    (cond ((zerop (length arg))
	   ;; no arg -- swap pwd and car of stack unless shell-pushd-tohome
	   (cond (shell-pushd-tohome
		  (shell-process-pushd (concat comint-file-name-prefix "~/")))
		 (shell-dirstack
		  (let ((old default-directory))
		    (shell-cd (car shell-dirstack))
		    (setq shell-dirstack
			  (cons old (cdr shell-dirstack)))
		    (shell-dirstack-message)))
		 (t
		  (message "Directory stack empty."))))
	  ((numberp num)
	   ;; pushd +n
	   (cond ((> num (length shell-dirstack))
		  (message "Directory stack not that deep."))
		 ((= num 0)
		  (error (message "Couldn't cd.")))
		 (shell-pushd-dextract
		  (let ((dir (nth (1- num) shell-dirstack)))
		    (shell-process-popd arg)
		    (shell-process-pushd dir)))
		 (t
		  (let* ((ds (cons default-directory shell-dirstack))
			 (dslen (length ds))
			 (front (nthcdr num ds))
			 (back (reverse (nthcdr (- dslen num) (reverse ds))))
			 (new-ds (append front back))
			 (new-dir (car new-ds)))
		    (shell-cd new-dir)
		    (setq shell-dirstack (cdr new-ds))
		    (shell-dirstack-message)))))
	  (t
	   ;; pushd <dir>
	   (let* ((old-wd default-directory)
		 (dir (shell-prefixed-directory-name arg))
		 (ex-dir (expand-directory-name dir)))
	     (if (not (string-match "/$" dir))
		 (setq dir (concat dir "/")))
	     (shell-cd dir)
	     (setq shell-dirstack (cons old-wd shell-dirstack))
	     (if (and shell-pushd-dunique
		      (member ex-dir
			      (mapcar 'expand-directory-name shell-dirstack)))
		 (setq shell-dirstack
		       (remove-matching-elements ex-dir shell-dirstack
						 'equal-dir-name))))
	   (shell-dirstack-message)))))


(defun my-shell-dirstack-message (&optional show)
  "Show the current dirstack on the message line.
If the variable shell-pushd-silent is non-nil, then no message
is shown.  This mimicks the optional behavior of tcsh."
  (if (and shell-pushd-silent (null show))
      nil
    (let* ((msg "")
	   (ds (cons default-directory shell-dirstack))
	   (home (expand-file-name (concat comint-file-name-prefix "~/")))
	   (homelen (length home)))
      (while ds
	(let ((dir (car ds)))
	  (and (>= (length dir) homelen) (string= home (substring dir 0 homelen))
	       (setq dir (concat "~/" (substring dir homelen))))
	  ;; Strip off comint-file-name-prefix if present.
	  (and comint-file-name-prefix
	       (>= (length dir) (length comint-file-name-prefix))
	       (string= comint-file-name-prefix
			(substring dir 0 (length comint-file-name-prefix)))
	       (setq dir (substring dir (length comint-file-name-prefix)))
	       (setcar ds dir))
	  (setq msg (concat msg (directory-file-name dir) " "))
	  (setq ds (cdr ds))))
      (message "%s" msg))))

(defun my-shell-replace-by-expanded-directory ()
  "Expand directory stack reference before point.
Directory stack references are of the form \"=digit\" or \"=-\".
See `default-directory' and `shell-dirstack'.

Now handles trailing / correctly.

Returns t if successful."
  (interactive)
  (if (comint-match-partial-filename)
      (save-excursion
	(goto-char (match-beginning 0))
	(let ((stack (cons default-directory shell-dirstack))
	      (index (cond ((looking-at "=-/?")
			    (length shell-dirstack))
			   ((looking-at "=\\([0-9]+\\)/?")
			    (string-to-number
			     (buffer-substring
			      (match-beginning 1) (match-end 1)))))))
	  (cond ((null index)
		 nil)
		((>= index (length stack))
		 (error "Directory stack not that deep."))
		(t
		 (replace-match (file-name-as-directory (nth index stack)) t t)
		 (message "Directory item: %d" index)
		 t))))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "File and Navigation Modes")                           ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Dired-mode

(defun my-dired-load-hook ()
  (defun dired-find-file-read-only ()
    "In dired, visit the file or directory named on this line in read-only state."
    (interactive)
    (let ((file-name (file-name-sans-versions (dired-get-filename) t)))
      (if (file-exists-p file-name)
          (find-file-read-only file-name)
        (if (file-symlink-p file-name)
            (error "File is a symlink to a nonexistent target")
          (error "File no longer exists; type `g' to update Dired buffer"))))
    )
  (defun dired-mac-open ()
    "In dired, run open command on given file or on each file in list"
    (interactive)
    (dired-do-shell-command "open")
    )
  (defalias 'dired-advertised-find-file-read-only 'dired-find-file-read-only)
  (setq dired-ls-F-marks-symlinks t) ;; Says ls -F marks symlinks with @
  (setq-default dired-listing-switches "-lF")
  )

(defun my-dired-mode-hook ()
  (local-set-key "\C-m" 'my-dired-find-file-or-subdir)
  (local-set-key "r" 'dired-advertised-find-file-read-only)
  (local-set-key "F" 'find-dired)
  (local-set-key "G" 'find-grep-dired)
  (local-set-key "N" 'find-name-dired)
  (local-set-key "%F" 'find-dired)
  (local-set-key "%G" 'find-grep-dired)
  (local-set-key "%N" 'find-name-dired)
  (local-set-key "W" 'browse-url-of-dired-file)
  (local-set-key "\M-o" 'dired-mac-open)
  (setq dired-listing-switches "-lF")
  )

(add-hook 'dired-load-hook 'my-dired-load-hook)
(add-hook 'dired-mode-hook 'my-dired-mode-hook)

(if (not (featurep 'dired+))
    (message "Dired+ is not loaded")
  (set-face-background 'diredp-no-priv     "white")
  (set-face-background 'diredp-write-priv  "tan")
  (set-face-background 'diredp-dir-heading "bisque1")
  (set-face-background 'diredp-dir-priv    "white")
  (set-face-foreground 'diredp-dir-priv    "#FF3445")
  (set-face-foreground 'diredp-file-name   "#3D34FF")
  (set-face-foreground 'diredp-file-suffix "#CF34FF")
  )


(defun my-dired-find-file-or-subdir ()
  "In dired, visit the file or sub-directory named on this line.
If the name corresponds to a sub-directory we use dired-maybe-insert-subdir;
otherwise, we use dired-find-file.  This allows directory navigation to all
occur within the same buffer. \\[dired-find-file] still works as before
if separate buffers are desired."
  (interactive)
  (let ((dir  (file-name-as-directory (dired-get-filename))))
    (if (not (file-directory-p dir))
        (dired-find-file)
      (dired-maybe-insert-subdir dir
                                 (if current-prefix-arg
                                     (read-string "Switches for listing: " dired-actual-switches)))
      (dired-goto-next-file))
    ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Linkd Mode

(require-soft 'linkd)
(setq linkd-use-icons t)
(setq linkd-icons-directory (concat my-site-lisp-dir "other-modes/linkd-icons"))
(setq linkd-wiki-directory "~/.emacs.d/linkd-wiki")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Icicles

(require 'icicles)
(add-to-list 'icicle-modal-cycle-up-keys   "\C-p")
(add-to-list 'icicle-modal-cycle-down-keys "\C-n")

(defun my-icicle-mode-hook ()
  )

;;(add-hook 'icicle-mode-hook 'my-icicle-mode-hook)

;;(defun my-icicle-minibuffer-setup-hook ()
;;  (define-key minibuffer-local-completion-map "\C-n" 'icicle-next-candidate-per-mode)
;;  (define-key minibuffer-local-completion-map "\C-p" 'icicle-previous-candidate-per-mode)
;;  )
;;(add-hook 'icicle-minibuffer-setup-hook 'my-icicle-minibuffer-setup-hook)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Interactive Do Things (ido-mode)

(require 'ido)
(defvar my-ido-begin-str "~/")
(defvar my-ido-begin-home 'home
  "*Determines the behavior of \\[my-ido-begin-line].
    If equal 'home,  replaces existing
    file text with the home directory. If 'root,
    replaces it with root. And if 'old, replaces
    it with value of \\[my-ido-begin-str]. The
    function toggles among these states.")

(defun my-ido-begin-line ()
  "*Moves to the beginning of the find file line,
    toggling between the home directory and root
    on successive calls."
  (interactive)
  (let
      ;; ATTN: Fix this 12 == length("Find file: ")
      ((contents (buffer-substring-no-properties 12 (minibuffer-prompt-end))))
    (cond
     ((equal my-ido-begin-home 'home)
      (insert "~/")
      (setq my-ido-begin-home 'root)
      ;;(setq my-ido-begin-str contents)
      (kill-new contents) ;;ATTN: for now make old string available to yank
      )
     ((equal my-ido-begin-home 'root)
      (insert "//")
      (setq my-ido-begin-home 'home) ;;'old)
      )
     (t ;; ATTN: for now this isn't working because delete fails so ignore it
      (setq my-ido-begin-home 'home)
      ;;(delete-region 12 (minibuffer-prompt-end))
      ;;(insert my-ido-begin-str)
      )))
  (setq ido-rescan t)
  )
  

(defun my-ido-setup-hook ()
  "Adjust keymaps for ido completion and other customizations."
  (define-key ido-file-completion-map "\C-a" 'my-ido-begin-line)
  (define-key ido-file-completion-map "\C-p" 'ido-prev-match)
  (define-key ido-file-completion-map "\C-n" 'ido-next-match)
  (define-key ido-file-completion-map "\C-xc"    'ido-toggle-case)
  (define-key ido-file-completion-map "\C-x\C-c" 'ido-toggle-case)
  (define-key ido-file-completion-map "\C-x\C-i" 'ido-toggle-ignore)
  (define-key ido-file-completion-map "\C-xi"    'ido-toggle-ignore)
  (define-key ido-file-completion-map "\C-xp"    'ido-toggle-prefix)
  (define-key ido-file-completion-map "\C-x\C-p" 'ido-toggle-prefix)
  (define-key ido-file-completion-map "\C-xr"    'ido-toggle-regexp)
  (define-key ido-file-completion-map "\C-x\C-r" 'ido-toggle-regexp)
  (setq ido-enable-flex-matching t)
  )

(add-hook 'ido-setup-hook 'my-ido-setup-hook)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Anything


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; YASnippet

(require 'yasnippet)
(setq yas/root-directory "~/.emacs.d/snippets")
(yas/initialize)
(yas/load-directory yas/root-directory)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; View-mode

(defun my-view-mode-hook ()
  (define-key view-mode-map "\M-v" 'end-of-buffer)
  (define-key view-mode-map "k"    'View-kill-and-leave)
  (define-key view-mode-map "l"    'View-leave)
  )

(add-hook 'view-mode-hook 'my-view-mode-hook)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Help-mode

(defun my-help-mode-hook ()
  (local-set-key ","    'rename-uniquely)
  (local-set-key "a"    'command-apropos)
  (local-set-key "b"    'describe-bindings)
  (local-set-key "c"    'describe-key-briefly)
  (local-set-key "C"    'describe-coding-system)
  (local-set-key "d"    'apropos-documentation)
  (local-set-key "e"    'view-echo-area-messages)
  (local-set-key "f"    'describe-function)
  (local-set-key "F"    'Info-goto-emacs-command-mode)
  (local-set-key "i"    'info)
  (local-set-key "k"    'describe-key)
  (local-set-key "K"    'Info-goto-emacs-key-command-mode)
  (local-set-key "l"    'view-lossage)
  (local-set-key "L"    'describe-language-environment)
  (local-set-key "m"    'describe-mode)
  (local-set-key "v"    'describe-variable)
  (local-set-key "p"    'finder-by-keyword)
  (local-set-key "q"    'View-kill-and-leave)
  (local-set-key "r"    'info-emacs-manual)
  (local-set-key "s"    'describe-syntax)
  (local-set-key "S"    'info-lookup-symbol)
  (local-set-key "w"    'where-is)
  )

(add-hook 'help-mode-hook 'my-help-mode-hook)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "TeX-related Modes")                                   ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; AUC TeX support

(setq TeX-default-mode 'plain-TeX-mode)
(setq TeX-format-list  '(("AMSTEX" ams-tex-mode "\\\\document\\b")
			 ("LATEX" latex-mode "\\\\\\(documentstyle\\|documentclass\\)\\b")
			 ("TEX" plain-tex-mode ".")
			 ))

(setq TeX-expand-list
      '(
       ("%p" TeX-printer-query)
       ("%q"
        (lambda nil
          (TeX-printer-query TeX-queue-command 2)))
       ("%v"
        (lambda nil
          (TeX-style-check TeX-view-style)))
       ("%V"
        (lambda nil
          (TeX-style-check TeX-output-view-style)))
       ("%r"
        (lambda nil
          (TeX-style-check TeX-print-style)))
       ("%l"
        (lambda nil
          (TeX-style-check LaTeX-command-style)))
       ("%s" file nil t)
       ("%t" file t t)
       ("%n" TeX-current-line)
       ("%d" file "dvi" t)
       ("%f" file "ps" t)
       ("%a" file "pdf" t)
       ("%b" TeX-current-file-name-nondirectory)) )

(setq TeX-command-list
  (list (list "TeX" "tex '\\nonstopmode\\input %t'" 'TeX-run-TeX nil t)
	(list "TeX Interactive" "tex %t" 'TeX-run-interactive nil t)
	(list "LaTeX" "%l '\\nonstopmode\\input{%t}'"
	      'TeX-run-LaTeX nil t)
	(list "LaTeX Interactive" "%l %t" 'TeX-run-interactive nil t)
	(list "LaTeX2e" "latex2e '\\nonstopmode\\input{%t}'"
	      'TeX-run-LaTeX nil t)
	(if (or window-system (getenv "DISPLAY"))
	    (list "View" "xdvi %d " 'TeX-run-silent t nil)
	  (list "View" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(if (or window-system (getenv "DISPLAY"))
	    (list "GhostView" "gv %f " 'TeX-run-silent t nil)
	  (list "GhostView" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(if (or window-system (getenv "DISPLAY"))
	    (list "NoteView" "gv --swap %f " 'TeX-run-silent t nil)
	  (list "NoteView" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(if (or window-system (getenv "DISPLAY"))
	    (list "AcroView" "acroread %a " 'TeX-run-silent t nil)
	  (list "AcroView" "dvi2tty -q -w 132 %s " 'TeX-run-command t nil))
	(list "File" "dvips %d -o %f " 'TeX-run-command t nil)
	(list "PDF" "dvipdf %d %a " 'TeX-run-command t nil)
	(list "Print" "%p " 'TeX-run-command t nil)
	(list "Queue" "%q" 'TeX-run-background nil nil)
	(list "BibTeX" "bibtex %s" 'TeX-run-BibTeX nil nil)
	(list "Index" "makeindex %s" 'TeX-run-command nil t)
	;; (list "Check" "chktex -v3 %s" 'TeX-run-compile nil t)
	;; Uncomment the above line and comment out the next line to
	;; use `chktex' instead of `lacheck'. 
	(list "Check" "lacheck %s" 'TeX-run-compile nil t)
	(list "Spell" "<ignored>" 'TeX-run-ispell-on-document nil nil)
	(list "Other" "" 'TeX-run-command t t)
	;; Not part of standard TeX.
	(list "Makeinfo" "makeinfo %t" 'TeX-run-compile nil t)
	(list "zAmSTeX" "amstex '\\nonstopmode\\input %t'"
	      'TeX-run-TeX nil t)) )


(defun my-plain-TeX-mode-hook ()
  (local-set-key "\C-c\C-e" 'LaTeX-environment)
  (local-set-key "$" 'self-insert-command)
  (setq LaTeX-default-environment "center")
  (setq TeX-font-list '( (?\C-b "{\\bf " "}")
			 (?\C-c "{\\sc " "}")
			 (?\C-e "{\\em " "\\/}")
			 (?\C-i "{\\it " "\\/}")
			 (?\C-m "{\\mb " "}")
			 (?\C-r "{\\rm " "}")
			 (?\C-s "{\\sl " "\\/}")
			 (?\C-t "{\\tt " "}")
			 (?\C-d "" "" t)
			 ))
  )
  
(add-hook 'plain-TeX-mode-hook 'my-plain-TeX-mode-hook)
(autoload 'LaTeX-environment "latex" "Make Environment Using AucTeX commands" t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; BibTeX-mode

(setq bibtex-mode-hook
      '((lambda ()
	  (local-set-key "\"" 'self-insert-command)
	  (setq bibtex-include-OPTannote nil)
	  (setq bibtex-include-OPTcrossref nil)
	  )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "ESS Mode for R and S Code")                           ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; ESS-mode for S 

;;
;; Basic Configuration
;;

(setq ess-fancy-comments nil)


;;
;; Formatting Style 
;;

(setq ESS-crg-style '(CRG (ess-indent-level . 4)
                          (ess-continued-statement-offset . 4)
                          (ess-continued-brace-offset . -4 )
                          (ess-brace-offset . 0)
                          (ess-brace-imaginary-offset . 0)
                          (ess-close-brace-offset . 0)
                          (ess-else-offset . 0)
                          (ess-expression-offset . 0)
                          (ess-arg-function-offset . 0)
                          (ess-fancy-comments . nil)
                          (ess-auto-newline . t)
                          (ess-tab-always-indent . t)
                   ))


(setq ess-mode-load-hook '((lambda ()
                             (if (not (assoc 'CRG ess-style-alist))
                                 (setq ess-style-alist (cons ESS-crg-style ess-style-alist)))
                             (setq ess-default-style 'CRG)
                             (setq ess-style 'CRG)
                             (setq ess-auto-newline t)
                             (setq-default ess-loop-timeout '1000000)
                             (setq-default inferior-ess-help-command "help(\"%s\")\n")
                             (setq ess-use-toolbar nil)
                             ;(progn
                             ;  (remove-hook 'ess-mode-hook             'turn-on-font-lock)
                             ;  (remove-hook 'ess-transcript-mode-hoonk 'turn-on-font-lock)
                             ;  (remove-hook 'inferior-ess-mode-hook    'turn-on-font-lock)
                             ;  )
                             (setq foofoobarbar '2) ;; just a test
                             )))

(setq inferior-ess-mode-hook '((lambda ()
                                 (local-set-key "\C-d" 'delete-char)
                                 (local-set-key "\C-c\M-p" 'comint-backward-matching-input)
                                 (local-set-key "\C-c\M-n" 'comint-forward-matching-input)
                                 (add-hook 'comint-input-filter-functions 'ess-search-path-tracker nil t)
                                 (setq input-ring-size '1024)
                                 (setq ess-style 'CRG)         ;; these three lines needed?
                                 (setq ess-auto-newline t)
                                 (ess-set-style)
                                 (setq inferior-ess-help-command "help(\"%s\")\n")
                                 ;;(progn
                                 ;;  (remove-hook 'ess-mode-hook             'turn-on-font-lock)
                                 ;;  (remove-hook 'ess-transcript-mode-hoonk 'turn-on-font-lock)
                                 ;;  (remove-hook 'inferior-ess-mode-hook    'turn-on-font-lock)
                                 ;;  )
                                 )))

(setq ess-pre-run-hook '((lambda ()
    (progn
      (remove-hook 'ess-mode-hook             'turn-on-font-lock)
      (remove-hook 'ess-transcript-mode-hoonk 'turn-on-font-lock)
      (remove-hook 'inferior-ess-mode-hook    'turn-on-font-lock)
      )
    )) )


(setq ess-mode-hook '((lambda ()
                        (setq ess-style 'CRG)
                        (setq ess-auto-newline t)
                        (ess-set-style)
                        )))

(setq ess-transcript-mode-hook '((lambda ()
                                   (setq ess-style 'CRG)
                                   (setq ess-auto-newline t)
                                   (ess-set-style)
                                   )))


;;
;; Load the functions
;;

(require 'ess-site)


;;
;; Disable Font-Lock support in ess-mode
;;

;;;;;Might want to try it for a while 2 Sep 2010
;;;(require 'ess)
;;;
;;;(progn
;;;  (remove-hook 'ess-mode-hook             'turn-on-font-lock)
;;;  (remove-hook 'ess-transcript-mode-hoonk 'turn-on-font-lock)
;;;  (remove-hook 'inferior-ess-mode-hook    'turn-on-font-lock)
;;;  )
;;;
;;;(setq inferior-ess-font-lock-input nil)  ;; needed?

;; (setq ess-style-alist (cons ESS-crg-style ess-style-alist)) ;;  needed?



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Global Key Bindings")                                 ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; ATTN: Want maps for
;;   point stuff   C-x p
;;   go map     M-g
;;   consider full reg-rect map 

;;
;; Auxilliary Global Keymaps
;;
    ;; String Replacement Commands

(define-prefix-command 'my-string-replace-prefix 'my-string-replace-map "String Replacement")
(define-key my-string-replace-map "q" 'query-replace)
(define-key my-string-replace-map "x" 'query-replace-regexp)
(define-key my-string-replace-map "s" 'replace-string)
(define-key my-string-replace-map "r" 'replace-regexp)
(define-key my-string-replace-map "t" 'tags-query-replace)

    ;; Tag Management Commands

(define-prefix-command 'my-tags-prefix 'my-tags-map "Tag Management")
(define-key my-tags-map "l"   'list-tags)
(define-key my-tags-map "."   'find-tag)
(define-key my-tags-map "f"   'find-tag)
(define-key my-tags-map "r"   'find-tag-regexp)
(define-key my-tags-map "4"   'find-tag-other-window)
(define-key my-tags-map "5"   'find-tag-other-frame)
(define-key my-tags-map "n"   'find-tag-noselect)
(define-key my-tags-map "c"   'tags-loop-continue)
(define-key my-tags-map "s"   'tags-search)
(define-key my-tags-map "\C-i" 'complete-tag)
(define-key my-tags-map "t"   'select-tags-table)
(define-key my-tags-map "v"   'visit-tags-table)
(define-key my-tags-map "z"   'tags-reset-tags-table)
(define-key my-tags-map "a"   'tags-apropos)
(define-key my-tags-map "q"   'tags-query-replace)

    ;; Marks and so forth (e.g., Bookmarks)

(defvar my-mark-map (copy-keymap bookmark-map) "Mark/Bookmark Management")
(fset 'my-mark-prefix my-mark-map)

    ;; Go Map  (Goto's and Jumps of All Kinds)

(define-prefix-command 'my-go-prefix 'my-go-map "Go")
(define-key my-go-map "b"    'bmkp-jump-map)
(define-key my-go-map "B"    'bmkp-jump-other-window-map)
(define-key my-go-map "g"    'goto-line)
(define-key my-go-map "\M-g" 'goto-line)
(define-key my-go-map "n"    'next-error)
(define-key my-go-map "\M-n" 'next-error)
(define-key my-go-map "p"    'previous-error)
(define-key my-go-map "\M-p" 'previous-error)

    ;; Abbrevs

(define-prefix-command 'my-abbrev-prefix 'my-abbrev-map "Abbreviations")
(define-key my-abbrev-map "d"      'define-global-abbrev)
(define-key my-abbrev-map "\C-d"   'define-mode-abbrev)
(define-key my-abbrev-map "m"      'add-mode-abbrev)
(define-key my-abbrev-map "g"      'add-global-abbrev)
(define-key my-abbrev-map "\C-m"   'inverse-add-mode-abbrev)
(define-key my-abbrev-map "\C-g"   'inverse-add-global-abbrev)
(define-key my-abbrev-map "e"      'expand-abbrev)
(define-key my-abbrev-map "r"      'expand-region-abbrevs)
(define-key my-abbrev-map "u"      'unexpand-abbrev)
(define-key my-abbrev-map " "      'set-abbrev-mark)
(define-key my-abbrev-map "v"      'edit-abbrevs)
(define-key my-abbrev-map "l"      'list-abbrevs)
(define-key my-abbrev-map "k"      'kill-all-abbrevs)



;;
;; Keybinding Setters
;;

(defun set-crg-global-sysdep-keybindings ()
  "Sets global keybindings that are system
   dependent (operating system or emacs version)."
   nil
  )

(defun set-crg-global-keybindings ()
  "Adjusts default global keybindings to
   conform to my preferences."
  ;;
  ;; Basic Operations
  ;;
    (global-set-key "\C-w"        'scroll-down-or-beg)
    (global-set-key "\C-v"        'scroll-up-or-end)
    (global-set-key "\M-v"        'end-of-buffer)
    (global-set-key "\M-w"        'beginning-of-buffer)
    (global-set-key "\M-k"        'kill-region)
    (global-set-key "\M-c"        'my-copy-region-as-kill)
    (global-set-key "\C-x\C-b"    'ibuffer-list-buffers)
    (global-set-key "\C-xL"       'count-lines-page)
    (global-set-key "\C-xl"       'what-line)
    (global-set-key "\C-xt"       'goto-line)
    (global-set-key "\C-xy"       'bury-buffer)
    (global-set-key "\C-xo"       'other-window)         ;; also bound to C-x C-o
    (global-set-key "\C-x\C-o"    'other-window)         ;; same to avoid collisions
    (global-set-key "\C-x\M-w"    'append-to-file)
    (global-set-key "\C-x\M-\C-f" 'find-file-at-point)
    (global-set-key "\M-o"        'delete-blank-lines )
    (global-set-key "\C-\\"       'save-buffer)  ;; For terminals with C-s wired as stop
  ;;
  ;; Rectangle and Register Commands
  ;;
    (global-set-key "\C-xra"      'append-to-register)
    (global-set-key "\C-xrc"      'copy-to-register)            
    (global-set-key "\C-xrl"      'list-registers)            
    (global-set-key "\C-xrp"      'prepend-to-register)
    (global-set-key "\C-xrs"      'string-rectangle)
    (global-set-key "\C-xrt"      'register-to-point)
    (global-set-key "\C-xrv"      'view-register)
    (global-set-key "\C-xrx"      'clear-rectangle)
    (global-set-key "\C-xr "      'point-to-register)
  ;;
  ;; Attach additional keymaps
  ;;
    (define-key ctl-x-map "g" 'my-tags-prefix)
    (define-key ctl-x-map "%" 'my-string-replace-prefix)
    (define-key ctl-x-map "m" 'my-mark-prefix)
    (define-key ctl-x-map "a" 'my-abbrev-prefix)
  ;;
  ;; Modified Arrow Keys get added functionality
  ;;
    (global-set-key [C-left]      'back-to-indentation)
    (global-set-key [C-right]     'end-of-line)
    (global-set-key [M-left]      'backward-word)
    (global-set-key [M-right]     'forward-word)
    (global-set-key [C-M-left]    'backward-list)
    (global-set-key [C-M-right]   'forward-list)
    (global-set-key [C-up]        'scroll-down-or-beg)
    (global-set-key [C-down]      'scroll-up-or-end)
    (global-set-key [M-up]        'backward-paragraph)
    (global-set-key [M-down]      'forward-paragraph)
    (global-set-key [C-M-up]      'backward-page)
    (global-set-key [C-M-down]    'forward-page)
  ;;
  ;; Search Keys
  ;;
    ;; To end a search with a specially defined key, like a keypad key
    ;; and to have the search not automatically continue with a single
    ;; C-s, do the following for the key.  For example, this makes
    ;; kp-2 act like a proper search ender.
    ;; (define-key isearch-mode-map [kp-2] 'isearch-other-control-char)
    (define-key isearch-mode-map "\C-w" 'isearch-other-control-char)
  ;;
  ;; System Dependent Global Bindings
  ;;
    (set-crg-global-sysdep-keybindings)
  ;;
  ;; Third-Party Mode Global Access Keys
  ;;
    (global-set-key "\C-cl"       'org-store-link)
    (global-set-key "\C-ca"       'org-agenda)
    (global-set-key [?\e ?\M-x] 'lacarte-execute-command)
    (global-set-key [?\M-`] 'lacarte-execute-command)
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Settings and Initialization")                         ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Start-up configuration

(setq inhibit-startup-message 't)
(setq initial-major-mode 'fundamental-mode)
(setq initial-frame-alist my-frame-alist)  

(setq custom-file (expand-file-name "~/.emacs.d/emacs-custom.el")) ;; load in init.el

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Enable Disabled Commands

(put 'eval-expression  'disabled nil)
(put 'narrow-to-region 'disabled nil)
(put 'downcase-region  'disabled nil)
(put 'upcase-region    'disabled nil)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Basic Operation

(defun set-crg-completion-mode (sym)
  "Choose among various intelligent completion methods
   according to symbol. Current choices are 'icicles, 'ido,
   'iswitchb, 'none."
   (cond
    ((equal sym 'icicles)
     (if (not (featurep 'icicles))
         (require 'icicles))
     (icicle-mode 1))
    ((equal sym 'ido)
     (ido-mode '1)
     (setq ido-case-fold t))
    ((equal sym 'iswitchb) ;;; Use smart buffer switching
     (iswitchb-mode '1)                      
     (setq iswitchb-case t))
    ))
   
    
(defun set-crg-operating-state ()
  "Set Emacs to preferred modes, maps, and more."
  (setq-default case-fold-search 'nil)
  (setq-default indent-tabs-mode nil)
    
  (setq-default fill-column '72)
  (setq paragraph-start paragraph-separate)            ;;; Use blank lines to separate paragraphs by default
  (setq adaptive-fill-regexp "[ \t]*\\([>*%#]+ +\\)?") ;;; Fill around comment beginnings and yanked-messages
    
  (setq print-length 1024)                 ;;; Give more information in help
  (setq print-level  8)

  (setq default-major-mode 'fundamental-mode)  ;; try org-mode but it causes strange behavior??
  
  (setq display-time-24hr-format t)
  (display-time)                           ;;; Time on Mode Line
    
  (transient-mark-mode '1)                 ;;; Highlight region
  (menu-bar-mode  -1)                      ;;; Disable menu bar
  (tool-bar-mode  -1)                      ;;; Disable tool bar
  (setq-default scroll-bar-mode 'right)    ;;; Put scroll bars on the right
  (toggle-scroll-bar  '1)                  ;;; Use scroll bars

  (set-crg-completion-mode 'icicles)
  (set-crg-global-keybindings)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "System Dependent Settings")                           ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(if (and (getenv "OSTYPE") (string-equal "linux" (downcase (getenv "OSTYPE"))))
    (progn
      (setq list-directory-verbose-switches "-HlrtF")
      (setq list-directory-brief-switches   "-CHvF")
      )
  (setq list-directory-verbose-switches "-lrtF")
  (setq list-directory-brief-switches   "-CF")
  )


(if (eq window-system 'x)
    (progn
      (set-face-background 'highlight "tan")   ;;; "red" is nice also
      (setq visible-bell 't)           
      ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Faces")                                               ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(if window-system
    (progn
      (set-face-background 'region "lightgoldenrod2")  
      (set-face-background 'secondary-selection "gray")
      )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Initial State")                                       ;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(load-file custom-file)       ;; customization-only customizations
(set-crg-operating-state)
(shell)
