;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "File and Navigation Modes")                           ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Dired-mode

(defun my-dired-load-hook ()
  (defun dired-find-file-read-only ()
    "In dired, visit the file or directory named on this line in read-only state."
    (interactive)
    (let ((file-name (file-name-sans-versions (dired-get-filename) t)))
      (if (file-exists-p file-name)
          (find-file-read-only file-name)
        (if (file-symlink-p file-name)
            (error "File is a symlink to a nonexistent target")
          (error "File no longer exists; type `g' to update Dired buffer"))))
    )
  (defun dired-mac-open ()
    "In dired, run open command on given file or on each file in list"
    (interactive)
    (dired-do-shell-command "open")
    )
  (defalias 'dired-advertised-find-file-read-only 'dired-find-file-read-only)
  (setq dired-ls-F-marks-symlinks t) ;; Says ls -F marks symlinks with @
  (setq-default dired-listing-switches "-lF")
  )

(defun my-dired-mode-hook ()
  (local-set-key "\C-m" 'my-dired-find-file-or-subdir)
  (local-set-key "r" 'dired-advertised-find-file-read-only)
  (local-set-key "F" 'find-dired)
  (local-set-key "G" 'find-grep-dired)
  (local-set-key "N" 'find-name-dired)
  (local-set-key "%F" 'find-dired)
  (local-set-key "%G" 'find-grep-dired)
  (local-set-key "%N" 'find-name-dired)
  (local-set-key "W" 'browse-url-of-dired-file)
  (local-set-key "\M-o" 'dired-mac-open)
  (setq dired-listing-switches "-lF")
  )

(add-hook 'dired-load-hook 'my-dired-load-hook)
(add-hook 'dired-mode-hook 'my-dired-mode-hook)

(if (not (featurep 'dired+))
    (message "Dired+ is not loaded")
  (set-face-background 'diredp-no-priv     "white")
  (set-face-background 'diredp-write-priv  "tan")
  (set-face-background 'diredp-dir-heading "bisque1")
  (set-face-background 'diredp-dir-priv    "white")
  (set-face-foreground 'diredp-dir-priv    "#FF3445")
  (set-face-foreground 'diredp-file-name   "#3D34FF")
  (set-face-foreground 'diredp-file-suffix "#CF34FF")
  )


(defun my-dired-find-file-or-subdir ()
  "In dired, visit the file or sub-directory named on this line.
If the name corresponds to a sub-directory we use dired-maybe-insert-subdir;
otherwise, we use dired-find-file.  This allows directory navigation to all
occur within the same buffer. \\[dired-find-file] still works as before
if separate buffers are desired."
  (interactive)
  (let ((dir  (file-name-as-directory (dired-get-filename))))
    (if (not (file-directory-p dir))
        (dired-find-file)
      (dired-maybe-insert-subdir dir
                                 (if current-prefix-arg
                                     (read-string "Switches for listing: " dired-actual-switches)))
      (dired-goto-next-file))
    ))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Linkd Mode

(require-soft 'linkd)
(setq linkd-use-icons t)
(setq linkd-icons-directory (concat my-site-lisp-dir "other-modes/linkd-icons"))
(setq linkd-wiki-directory "~/.emacs.d/linkd-wiki")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Icicles

(require-soft 'icicles)
(when (featurep 'icicles)
  (add-to-list 'icicle-modal-cycle-up-keys              "\C-p")
  (add-to-list 'icicle-modal-cycle-up-action-keys       "\C-\M-p")
  (add-to-list 'icicle-modal-cycle-up-alt-action-keys   [?\C-\S-p])
  (add-to-list 'icicle-modal-cycle-up-help-keys         [?\C-\M-\S-p])
  (add-to-list 'icicle-modal-cycle-down-keys            "\C-n")
  (add-to-list 'icicle-modal-cycle-down-action-keys     "\C-\M-n")
  (add-to-list 'icicle-modal-cycle-down-alt-action-keys [?\C-\S-n])
  (add-to-list 'icicle-modal-cycle-down-help-keys       [?\C-\M-\S-n])
  )

(defvar my-icicle-describe-sort-in-completions (and t (featurep 'icicles))
  "If non-nil, show the current sort order in *Completions* buffer.")

(defun my-icicle-show-sort-order ()
  "Show current sort order in completions list.
   Add this to completion-setup-hook to achieve
   the effect."
  (when (and my-icicle-describe-sort-in-completions icicle-mode)
    (with-current-buffer standard-output
      (let* ((sort-order (icicle-current-sort-order nil))
             (sort-string (concat ", sorting "
                                  (if (string-equal sort-order "alphabetical")
                                      "alphabetically" sort-order))))
        (goto-char (point-min))
        (when (re-search-forward "Possible completions are" nil t)
          (put-text-property 0 (length sort-string) 'face 'icicle-Completions-instruction-1 sort-string)
          (insert sort-string)))))
  )

(defun my-icicle-mode-hook ()
  (setq read-file-name-completion-ignore-case t) ; Emacs >= 22
  (setq read-buffer-completion-ignore-case t)    ; Emacs >= 23
  (setq icicle-show-Completions-help-flag nil)
  (if icicle-mode
      (add-hook 'completion-setup-hook 'my-icicle-show-sort-order t)
    (remove-hook 'completion-setup-hook 'my-icicle-show-sort-order))
  )

(add-hook 'icicle-mode-hook 'my-icicle-mode-hook)

(defun my-icicle-minibuffer-setup-hook ()
  (define-key minibuffer-local-completion-map "\C-x\C-o" 'icicle-switch-to/from-minibuffer)
  ;(define-key minibuffer-local-completion-map "\C-n" 'icicle-next-candidate-per-mode)
  ;(define-key minibuffer-local-completion-map "\C-p" 'icicle-previous-candidate-per-mode)
  )
(add-hook 'icicle-minibuffer-setup-hook 'my-icicle-minibuffer-setup-hook)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Interactive Do Things (ido-mode)

(require-soft 'ido)
(defvar my-ido-begin-str "~/")
(defvar my-ido-begin-home 'home
  "*Determines the behavior of \\[my-ido-begin-line].
    If equal 'home,  replaces existing
    file text with the home directory. If 'root,
    replaces it with root. And if 'old, replaces
    it with value of \\[my-ido-begin-str]. The
    function toggles among these states.")

(defun my-ido-begin-line ()
  "*Moves to the beginning of the find file line,
    toggling between the home directory and root
    on successive calls."
  (interactive)
  (let
      ;; ATTN: Fix this 12 == length("Find file: ")
      ((contents (buffer-substring-no-properties 12 (minibuffer-prompt-end))))
    (cond
     ((equal my-ido-begin-home 'home)
      (insert "~/")
      (setq my-ido-begin-home 'root)
      ;;(setq my-ido-begin-str contents)
      (kill-new contents) ;;ATTN: for now make old string available to yank
      )
     ((equal my-ido-begin-home 'root)
      (insert "//")
      (setq my-ido-begin-home 'home) ;;'old)
      )
     (t ;; ATTN: for now this isn't working because delete fails so ignore it
      (setq my-ido-begin-home 'home)
      ;;(delete-region 12 (minibuffer-prompt-end))
      ;;(insert my-ido-begin-str)
      )))
  (setq ido-rescan t)
  )

(defun my-ido-setup-hook ()
  "Adjust keymaps for ido completion and other customizations."
  (define-key ido-file-completion-map "\C-a" 'my-ido-begin-line)
  (define-key ido-file-completion-map "\C-p" 'ido-prev-match)
  (define-key ido-file-completion-map "\C-n" 'ido-next-match)
  (define-key ido-file-completion-map "\C-xc"    'ido-toggle-case)
  (define-key ido-file-completion-map "\C-x\C-c" 'ido-toggle-case)
  (define-key ido-file-completion-map "\C-x\C-i" 'ido-toggle-ignore)
  (define-key ido-file-completion-map "\C-xi"    'ido-toggle-ignore)
  (define-key ido-file-completion-map "\C-xp"    'ido-toggle-prefix)
  (define-key ido-file-completion-map "\C-x\C-p" 'ido-toggle-prefix)
  (define-key ido-file-completion-map "\C-xr"    'ido-toggle-regexp)
  (define-key ido-file-completion-map "\C-x\C-r" 'ido-toggle-regexp)
  (setq ido-enable-flex-matching t)
  )

(add-hook 'ido-setup-hook 'my-ido-setup-hook)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Anything


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; YASnippet

(require-soft 'yasnippet)

(when (featurep 'yasnippet)
  (setq yas/root-directory "~/.emacs.d/snippets")
  (yas/initialize)
  (yas/load-directory yas/root-directory)
  (setq yas/prompt-functions '(yas/ido-prompt yas/dropdown-prompt yas/completing-prompt yas/x-prompt yas/no-prompt)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Which-Function

(require-soft 'which-func)
(setq which-func-modes 
      '(emacs-lisp-mode c-mode c++-mode python-mode perl-mode cperl-mode makefile-mode sh-mode fortran-mode f90-mode ada-mode))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; View-mode

(defun my-view-mode-hook ()
  (define-key view-mode-map "\M-v" 'end-of-buffer)
  (define-key view-mode-map "k"    'View-kill-and-leave)
  (define-key view-mode-map "l"    'View-leave)
  )

(add-hook 'view-mode-hook 'my-view-mode-hook)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Help-mode

(defun my-help-mode-hook ()
  (local-set-key ","    'rename-uniquely)
  (local-set-key "a"    'command-apropos)
  (local-set-key "\M-a" 'apropos-documentation)
  (local-set-key "b"    'describe-bindings)
  (local-set-key "c"    'describe-key-briefly)
  (local-set-key "C"    'describe-coding-system)
  (local-set-key "d"    'apropos-documentation)
  (local-set-key "e"    'view-echo-area-messages)
  (local-set-key "f"    'describe-function)
  (local-set-key "F"    'Info-goto-emacs-command-node)
  (local-set-key "i"    'info)
  (local-set-key "k"    'describe-key)
  (local-set-key "K"    'Info-goto-emacs-key-command-node)
  (local-set-key "l"    'help-go-back) ; I just always want to do this, so...
  (local-set-key "L"    'view-lossage) ; minor inconsistency won't hurto
  (local-set-key "\M-l" 'locate-library)
  (local-set-key "m"    'describe-mode)
  (local-set-key "p"    'finder-by-keyword)
  (local-set-key "q"    'View-leave)
  (local-set-key "\M-q" 'View-kill-and-leave)
  (local-set-key "r"    'info-emacs-manual)
  (local-set-key "s"    'describe-syntax)
  (local-set-key "S"    'info-lookup-symbol)
  (local-set-key "u"    'manual-entry)
  (local-set-key "v"    'describe-variable)
  (local-set-key "V"    'describe-language-environment) ; another minor inconsistency
  (local-set-key "w"    'where-is)
  (local-set-key [\M-left] 'help-go-back)
  )

(add-hook 'help-mode-hook 'my-help-mode-hook)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Hippie Expand

(condition-case nil
    (delete 'try-expand-line hippie-expand-try-functions-list)
   ;(delete 'try-expand-list hippie-expand-try-functions-list)
   (error nil))
