;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Imported Mode Requirements")                          ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defun import (feature-list &optional action &rest xargs)
  "Apply ACTION to each of features given in FEATURE-LIST,
   supplying additional arguments if necessary. Typical
   use case is for ACTION to be require, require-soft,
   or autoload. If not supplied, ACTION defaults to
   require.

   Elements of FEATURE-LIST can have several forms.
   The feature can one of the following forms:

       1. a SYMBOL,
       2. a cons (CONDITION . SYMBOL),
       3. a list (SYMBOL ARGS...), or
       4. a list (CONDITION SYMBOL ARGS...).

   Here, SYMBOL is a symbol corresponding to a feature,
   typically one that can be validly require'd.
   And CONDITION is an expression that eval's
   to a boolean (validly at load and compile time)
   and is used to selectively load particular features.
   A typical example would be (>= emacs-major-version 22).
   ARGS is a list of 0 or more additional arguments.
   (In cases 1 and 2 above, ARGS will effectively be an
   an empty list.) The lists ARGS and that supplied in
   the &rest argument XARGS will be concatenated, in
   that order, and the corresponding elements will be
   passed as arguments to the ACTION function."
  (let ((req (or action 'require)))
    (dolist (feature feature-list)
      (let (sym args)
        (cond
         ;; Feature is a symbol
         ((symbolp feature)
          (setq sym feature)
          (setq args xargs)
          )
         ;; Feature is (list SYMBOL ARGS...)
         ((and (consp feature)
               (symbolp (car feature)))
          (setq sym (car feature))
          (setq args (concatenate 'list (cdr feature) xargs))
          )
         ;; Feature is (CONDITION . SYMBOL) or (list CONDITION SYMBOL ARGS...)
         ((and (consp feature)
               (booleanp (eval (car feature)))
               (or (symbolp (cdr feature))
                   (and (listp (cdr feature))
                        (symbolp (cadr feature)))) )
          (when (eval (car feature))
            (if (symbolp (cdr feature))  ;; have cons (CONDITION . SYMBOL)
                  (progn
                    (setq sym (cdr feature))
                    (setq args xargs)
                    )                    ;; else have (list CONDITION SYMBOL ...)
              (setq sym (cadr feature))
              (setq args (concatenate 'list (cddr feature) xargs))
              ))
          )
         (t
          (message (concat "Erroneous feature specification: " (prin1-to-string feature)))
          ))
        (if sym (apply req sym args))
        ))
    ))

(defalias 'deferred-import 'identity
  "Syntactic sugar to allow user to saliently list
   those modules that will be imported later in the
   init file but not in the main import list. This
   is usually done for timing purposes, for instance
   to ensure that hooks are properly defined at
   load time.")

;;
;; Lists of Packages to Import
;;

(defvar my-must-features
  '(comint shell calc)
  "*List of *built-in* features that must be required for later
   processing in this initialization file. Each element is either
   a symbol that is a valid argument to require or is a cons-cell
   whose car is a major version number for Emacs and whose cdr is
   a symbol that is a valid argument to require. These features
   will be required, and failure of the require will lead to an
   error.")

(defvar my-ought-features
  '(org-install icicles ido dired-x dired-aux)
  "*List of features that are distributed with emacs that would be
   desireable to require for later processing in this initi file. 
   If for some reason, they are not available, a message will be
   displayed but no error produced.

   Each element of the list is either a symbol that is a valid
   argument to require or is a cons-cell whose car is a major
   version number for Emacs and whose cdr is a symbol that is a
   valid argument to require.")

(defvar my-want-features
  '(dired+ bookmark+ linkd lacarte dropdown-list yasnippet anything-config ipython)
  "*List of features, not necessarily available, that it would be 
   desireable to require for later processing in this init file. 
   If for some reason, they are not available, a message will be
   displayed but no error produced.

   Each element of the list is either a symbol that is a valid
   argument to require or is a cons-cell whose car is a major
   version number for Emacs and whose cdr is a symbol that is a
   valid argument to require.")

(defvar my-autoloaded-features
  (list '(nxml-mode   "nxml/autostart.el" "nXML Mode"   t)
        '(nxhtml-mode "nxml/autostart.el" "nXHTML Mode" t)
        )
  "*List of features to be autoloaded along with the arguments
   to be passed to autoload. Elements of the lists are of
   the form (SYMBOL ARGS...).")

(defvar my-special-features
  (list
   (list 'ess-site   '(deferred-import "Temporarily deferring load to ess.el for timing purposes"))
   (list 'thingatpt+ '(eval-after-load "thingatpt"
                        '(require-soft 'thingatpt+)))
   (list 'cedet      '(if (not (featurep 'cedet))
                          (load-file (concat my-site-lisp-dir "cedet/common/cedet.el"))))
   (list 'jde        '(progn
                        (setq defer-loading-jde t)
                        (if (not defer-loading-jde)
                            (require-soft 'jde)
                          (autoload 'jde-mode "jde" "JDE mode." t)
                          (add-to-list 'auto-mode-alist '("\\.java\\'" . jde-mode))
                          )))
   (list 'ecb        '(progn
                        (eval-when-compile
                          (if (not (featurep 'cedet))
                              (require 'cedet)))
                        (require-soft 'ecb)
                        ))
   )
  "List of features that will need special processing (usually beyond
   a simple require) for use in this init file. Although somewhat
   awkward, the code to handle these cases should be given as
   a form in the list. Specifically, each element of the list
   should in turn be a list with either two or three elements
   corresponding to 

      (list 'SYMBOL 'BODYFORM) 
      (list  CONDITION 'SYMBOL 'BODYFORM),

   where CONDITION is a lisp expression that evaluates to t
   if this feature should be acted upon. A typical example
   for condition would be (>= emacs-major-version 22).
   In any case, CONDITION must be valid for evaluation at
   either load or compile time.")


;;
;; Do Imports and Autoloads
;;

(import my-must-features       'require)
(import my-ought-features      'require-soft)
(import my-want-features       'require-soft)
(import my-autoloaded-features 'autoload)
(import my-special-features    (lambda (sym body) (eval body)))


;;
;; Automatic Mode Detection
;;

(add-to-list 'auto-mode-alist '("\\.org\\'" . org-mode))
(add-to-list 'auto-mode-alist '("\\.txt\\'" . org-mode))
(add-to-list 'auto-mode-alist '("\\.[0-9]\\'" . fundamental-mode))
(add-to-list 'auto-mode-alist '("\\.[sS]\\'" . S-mode))
(add-to-list 'auto-mode-alist '("\\.[rR]\\'" . R-mode))
(add-to-list 'auto-mode-alist '("\\.\\([pP]\\([LlmM]?\\|od\\|erl\\)\\|al\\)\\'" . perl-mode))
(add-to-list 'auto-mode-alist 
             (cons (concat "\\." (regexp-opt '("xml" "xsd" "sch" "rng" "xslt" "svg" "rss") t) "\\'")
                   'nxml-mode))
(add-to-list 'auto-mode-alist '("\\.s?html?\\'" . nxhtml-mode))

(fset 'xml-mode 'nxml-mode)    ;; should rhs be (symbol-function 'nxml-mode) ??
(fset 'html-mode 'nxhtml-mode)

(add-to-list 'magic-mode-alist '("<\\?xml " . nxml-mode))
(add-to-list 'magic-mode-alist '("#!.*perl" . perl-mode))
(add-to-list 'magic-mode-alist '("#!.*python" . python-mode))
(add-to-list 'magic-mode-alist '("#!.*\\(t?c\\|ba\\|k\\|z\\|\\)sh" . shell-script-mode))
(add-to-list 'magic-mode-alist '("\\\\input " . plain-TeX-mode))
(add-to-list 'magic-mode-alist '("\\\\document\\(style\\|class\\)" . LaTeX-mode))


