;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                           ;;
;;(@* "Emacs Lisp Utility Functions")                        ;;;;;;
;;                                                           ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; Setting Functions for convenient adding to hooks

;(defun turn-on-auto-fill ()
;  (auto-fill-mode '1))

(defun turn-on-tex-parser-ispell ()
  (make-local-variable 'ispell-parser)
  (setq ispell-parser 'tex))

(defun turn-on-local-show-paren-mode ()
  (make-local-variable 'show-paren-mode)
  (setq show-paren-delay 0)  ; default 0.125
  (setq show-paren-style 'parenthesis)
  (show-paren-mode '1)
  (set-face-background 'show-paren-mismatch "red"))

(defun turn-on-global-show-paren-mode ()
  (setq show-paren-delay 0)  ; default 0.125
  (setq show-paren-style 'parenthesis)
  (show-paren-mode '1)
  (set-face-background 'show-paren-mismatch "red"))

(defun turn-on-local-comment-auto-fill ()
  (set (make-local-variable 'comment-auto-fill-only-comments) t)
  (auto-fill-mode '1))

(defun highlight-attn-words ()
  (font-lock-add-keywords
   nil '(("\\<\\(ATTN\\|TODO\\|FIX\\|FIXME\\|HACK\\):?"
          1 font-lock-warning-face t))))


;; Operations Functions

(defun make-backup-file-name (file)
  "Create the non-numeric backup file name for FILE.
This is a separate function so you can redefine it for customization."
  (concat (file-name-directory file) (concat "." (concat (file-name-nondirectory file) "~"))))

(defun set-abbrev-mark ()
  "Mark current point as the beginning of an abbrev.
Abbrev to be expanded starts here rather than at beginning of word before point."
  (interactive)
  (setq abbrev-start-location (point-marker)
	abbrev-start-location-buffer (current-buffer))
  (message "Abbrev Mark Set"))


;; Frame functions

(defun my-other-frame (arg)
  "Select the ARG'th different visible frame, and raise it, even if
the frame is iconified or invisible. All frames are arranged in a
cyclic order, and this command selects the frame ARG steps away in
that order.  A negative ARG moves in the opposite order."
  (interactive "p")
  (let ((frame (selected-frame)))
    (while (> arg 0)
      (setq frame (next-frame frame))
      (setq arg (1- arg)))
    (while (< arg 0)
      (setq frame (previous-frame frame))
      (setq arg (1+ arg)))
    (raise-frame frame)
    (select-frame frame)
    (set-mouse-position (selected-frame) (1- (frame-width)) 0)
    (if (fboundp 'unfocus-frame)
	(unfocus-frame))))


;; Scrolling and Cursor Movement functions

(defun scroll-down-or-beg ()
  "Smart scroll down, if too close to the beginning of the buffer,
it moves you there without error, in contrast to scroll-down."
  (interactive)
  (condition-case nil (scroll-down) (error (goto-char (point-min)))))

(defun scroll-up-or-end ()
  "Smart scroll up, if too close to the end of the buffer,
it moves you there without error, in contrast to scroll-up."
  (interactive)
  (condition-case nil (scroll-up) (error (goto-char (point-max)))))

(defun forward-sexp-or-char (&optional arg)
  "Move forward across one balanced expression (sexp) or forward
one character if none. With ARG, do it that many times.  Negative
arg -N means move backward across N balanced expressions. In
comparison to backward-sexp, this allows you to continue moving
across expressions without having to change keys."
  (interactive "p")
  (or arg (setq arg 1))
  (condition-case nil
      (forward-sexp arg)
    (error
     (let ((at-sexp-boundary? ; do we see a sexp delimiter in proper direction?
            (if (< arg 0)
                (if (char-equal (char-syntax (char-before (point))) ?\() t nil)
              (if (char-equal (char-syntax (char-after (point))) ?\)) t nil))))
       (if at-sexp-boundary?
           (forward-char arg))))))

(defun backward-sexp-or-char (&optional arg)
  "Move backward across one balanced expression (sexp) or
backward one character if at the boundary.  With ARG, do it that
many times.  Negative arg -N means move forward across N balanced
expressions.  In comparison to backward-sexp, this allows you to
continue moving across expressions without having to change
keys."
  (interactive "p")
  (or arg (setq arg 1))
  (forward-sexp-or-char (- arg)))


;; Region and Buffer Functions

  ;; Standard version is silent, so it is hard to tell if the key was hit
  ;; Only added a message

(defun my-copy-region-as-kill (beg end)
  "Save the region as if killed, but don't kill it.
If `interprogram-cut-function' is non-nil, also save the text for a window
system cut and paste.  Produces a message to confirm copy."
  (interactive "r")
  (if (eq last-command 'kill-region)
      (kill-append (buffer-substring beg end) (< end beg))
    (kill-new (buffer-substring beg end)))
  (message "Region Copied")
  nil)

(defun eval-and-replace-sexp ()
  "Replace the preceding emac-lisp sexp in current buffer
   with its value."
  (interactive)
  (backward-kill-sexp)
  (condition-case nil
      (prin1 (eval (read (current-kill 0)))
             (current-buffer))
    (error (message "Invalid expression")
           (insert (current-kill 0)))))


;; Import and Import-Conditional Execution

;; ATTN: macro version not appropriate because we need to 'apply this
;(defmacro require-soft (feature &optional file)
;  "Try to require FEATURE, but don't signal an error if `require' fails."
;  `(require ,feature ,file 'noerror)
;  )

(defun require-soft (feature &optional file)
  "Try to require FEATURE, but don't signal an error if `require' fails."
  (require feature file 'noerror))

(defmacro with-library (symbol &rest body)
   "Execute BODY conditionally on the availability of library SYMBOL"
   `(if (not (require ,symbol nil t))
        (message (format "Library %s is not available." ',symbol))
      ,@body))


;; List Matching Functions

(defun remove-matching-elements (elt lst &optional pred)
  "Remove elements in LST matching ELT according to the
predicate PRED. If PRED is not supplied, it defaults to equal."
  (let ((pred (or pred (symbol-function 'equal)))
	(newlst nil)
	cmp)
    (while lst
      (setq cmp (car lst))
      (if (not (funcall pred elt cmp))
	  (setq newlst (cons cmp newlst)))
      (setq lst (cdr lst)))
    (reverse newlst)))
      
(defun remove-non-matching-elements (elt lst &optional pred)
  "Remove elements in LST matching ELT according to the
predicate PRED. If PRED is not supplied, it defaults to equal."
  (let ((pred (or pred (symbol-function 'equal)))
	(newlst nil)
	cmp)
    (while lst
      (setq cmp (car lst))
      (if (funcall pred elt cmp)
	  (setq newlst (cons cmp newlst)))
      (setq lst (cdr lst)))
    (reverse newlst)))


;; Predicates for element removal
      
(defun eq-car (l1 l2)
  "Returns t if car of l1 and l2 are eq, nil otherwise.
If l1 (or l2) are not cons cells, l1 (or l2) itself is used in the comparison.
This is useful as a predicate for removing elements from an association list;
see remove-matching-elements.  In this case, if the keys are symbols then
only the symbol of interest need be passed to remove-matching elements."
  (eq (if (consp l1) (car l1) l1) (if (consp l2) (car l2) l2)))

(defun equal-car (l1 l2)
  "Returns t if car of l1 and l2 are equal, nil otherwise.
If l1 (or l2) are not cons cells, l1 (or l2) itself is used in the comparison.
This is useful as a predicate for removing elements from an association list;
see remove-matching-elements.  In this case, if the keys are symbols then
only the symbol of interest need be passed to remove-matching elements."
  (equal (if (consp l1) (car l1) l1) (if (consp l2) (car l2) l2)))


;; File and Directory Name Functions and Predicates

(defun expand-directory-name (dir)
  "Expands a directory name as in expand-file-name but ensures
that the name ends with a '/'.  This can be used to convert
a directory name, even in relative or ~ form, to a canonical
form, for comparison, concatenation, or other purposes."
  (save-excursion
    (let ((dir (and (stringp dir) (expand-file-name dir))))
      (if (and dir (not (string-match "/$" dir)))
	  (concat dir "/")
	dir))) )

(defun equal-file-name (fn1 fn2 &optional func)
  "Tests whether two file-names FN1 and FN2 refer to the same file.
This uses FUNC to bring both file-names to a common form (e.g., abolute).
If FUNC is not provided, it defaults to a expand-file-name."
  (if (and (stringp fn1) (stringp fn2))
      (let ((func (or func (symbol-function 'expand-file-name))))
	(string-equal (funcall func fn1) (funcall func fn2)))))

(defun equal-dir-name (fn1 fn2 &optional func)
  "Tests whether two file-names FN1 and FN2 refer to the same file.
This uses FUNC to bring both file-names to a common form (e.g., abolute).
If FUNC is not provided, it defaults to a function that does
expand-file-name followed by appending a trailing slash."
  (if (and (stringp fn1) (stringp fn2))
      (let ((func (or func (symbol-function 'expand-directory-name))))
	(string-equal (funcall func fn1) (funcall func fn2)))))


;; Miscellaneous Useful Functions and Tools

(defun my-get-makefile-targets ()
  "Scan the contents of all files makefiles in the current
directory and return a list of targets.  The files scanned match
the glob [mM]akefile* within .  All targets are returned except
those containing $ (for variable substitution) or % for pattern
matching."
  (with-temp-buffer
    (let ((makefiles (nreverse (file-expand-wildcards "[mM]akefile*")))
          targets)
      (dolist (file makefiles)
        (insert-file-contents (concat default-directory file)))
      (goto-char (point-min))
      ; Simply exclude $ and %. Can add more generality later if needed.
      (while (re-search-forward "^\\s-*\\([^$%: 	\n]+?\\)\\s-*:" nil t)
        (setq targets (cons (match-string 1) targets)))
      (nreverse (cons "" targets)))))


(defun view-url-in-buffer ()
  "Open a new buffer containing the contents of URL. If thingatpt+
   is loaded, tries to find URL nearest point as default; otherwise,
   no default is given."
  (interactive)
  (let* ((default (if (featurep 'thingatpt+) (thing-nearest-point 'url) "")) 
         (url (read-from-minibuffer "URL: " default))
         (urlbuf (url-retrieve-synchronously url)))
    (when urlbuf
      (switch-to-buffer urlbuf)
      (rename-buffer url t)
      (goto-char (point-min))
      ;; For an http request, header is included in the buffer,
      ;; so remove up to and including first blank line
      (when (and (looking-at "HTTP")
                 (re-search-forward "^\\s-*$" nil t))
        (condition-case nil (forward-char 1) (error nil)) ; move past blank line
        (delete-region (point-min) (point)))
      (setq case-fold-search t) ; buffer local
      ;; This should use nxml-mode or nxhtml-mode automatically
      (cond ((looking-at "<?xml") (xml-mode))
            ((looking-at "<\\(!DOCTYPE\\s-+\\)html") (html-mode))))))


